/* 
 Functions used within the sketch
*/

// *********************
// *** WIFI connect  ***
// *********************
void connectLocalWiFi() {
	unsigned long timeout= millis() +20000; // max time to try to connect to WiFi
	DPRINTLN("****************************************************");
	DPRINT("Connect to: ");	DPRINTLN(PRIVATE_WIFI_SSID);
	WiFi.begin(PRIVATE_WIFI_SSID, PRIVATE_WIFI_PASSWORD);
	while ( (WiFi.status() != WL_CONNECTED) && (timeout - millis() >0) ) {
		delay(500);
		DPRINT(".");
	}
	DPRINTLN();
	if (timeout - millis() < 0) {
		DPRINTLN("WiFi connection TIMEOUT!");
	}
	else {
		DPRINTLN("WiFi connected!");
		DPRINT("IP-adres: "); DPRINTLN(WiFi.localIP());
		DPRINT("Signal strength: "); DPRINTLN(WiFi.RSSI());
	}
}
// *********************
// ***  TIME SERVER  ***
// *********************
void getLocalDateTime() {

	String inputLine = "";
	bool dataFound = 0;
	int sepPos1 = 0;
	int sepPos2 = 0;
	int lenTot =0;
	
	TimeServerData = false;
	// Use WiFiClient class to create TCP connections to TIME SERVER
	DPRINT("Setting up TCP connection with TIME SERVER: ");DPRINTLN(hostT);
	clientS.setInsecure();
	if (!clientS.connect(hostT, httpsPort)) {
		Serial.println("Connection failed!!!");
	}
	else {
		DPRINT("Request url: ");DPRINTLN(PRIVATE_URLT);
		clientS.println("GET " + String(PRIVATE_URLT) + " HTTP/1.1");
		clientS.println("Host: " + String(hostT));
		clientS.println("Connection: close");
		clientS.println(); // this signals end of header to server
		
		// give the server some time to respond
		unsigned long timeout = millis() + 5000;
		while (clientS.available() == 0) {
			if (timeout - millis() < 0) {
				Serial.println("Client Timeout!!!");
				clientS.stop();
			}
		}
	}
	// process the time server response
	// Find the line with the DATE info, format "yyyy-mm-dd"
	dataFound = 0;
	while (clientS.available()&& dataFound == 0) {
		inputLine = clientS.readStringUntil('\r\n');
		//DPRINTLN("Date header inputLine: " + inputLine);            
		if (inputLine.indexOf("yyyy-mm-dd=") != -1) { 
			dataFound = 1;
			sepPos1 = inputLine.indexOf("=");
			sepPos2 = inputLine.indexOf("<br />");
			CurrentDate = inputLine.substring(sepPos1+1,sepPos2);
			DPRINTLN("CurrentDate: " + CurrentDate);
		}
	}
	// Find the line with the TIME info, format "hh:mm:ss" 24 hour notation, corrected for DST
	dataFound = 0;
	while (clientS.available()&& dataFound == 0) {
		inputLine = clientS.readStringUntil('\r\n');
		//DPRINTLN("Time header inputLine: " + inputLine);            
		if (inputLine.indexOf("hh:mm:ss=") != -1) { 
			dataFound = 1;
			sepPos1 = inputLine.indexOf("=");
			sepPos2 = inputLine.indexOf("<br />");
			CurrentTime = inputLine.substring(sepPos1+1,sepPos2);
			DPRINTLN("CurrentTime: " + CurrentTime);
			TimeServerData = true;
		}
	}
	// Empty the html input buffer
	while (clientS.available()) int x = clientS.read();
}
// *************************************
// *** send a command to the MAX7221 ***
// *************************************
// command contains 'address' and 'data'
// 'address' determines the operation of the MAX7221
void maxTransfer(byte address, byte value) {
	// Ensure SS is LOW
	digitalWrite(SS, LOW);
	// Send the register address
	SPI.transfer(address);
	// Send the value
	SPI.transfer(value);
	// Tell chip to load in data
	digitalWrite(SS, HIGH);
}
// ***********************************
// *** write the time to the clock ***
// ***********************************
// dp2 is on if WiFi is connected
// dp1 is on if time server data is received
void updateDisplay() {
	byte dig0;
	byte dig1;
	byte dp1 = MAX7221_DP_ON;
	byte dig2;
	byte dp2 = MAX7221_DP_ON;
	byte dig3;
	dig3 = ClockHour / 10;
	if (dig3==0) {
		// blank leading zero
		maxTransfer(MAX7221_DIGIT3,MAX7221_DP_OFF + 0x0F);
	} else {
		maxTransfer(MAX7221_DIGIT3,MAX7221_DP_OFF + dig3);
	}
	dig2 = ClockHour - dig3*10;
	if (WiFi.status() != WL_CONNECTED) dp2 = MAX7221_DP_OFF;
	maxTransfer(MAX7221_DIGIT2,dp2 + dig2);
	dig1 = ClockMin / 10;
	if ( (WiFi.status() != WL_CONNECTED) || (!TimeServerData) ) dp1 = MAX7221_DP_OFF;
	maxTransfer(MAX7221_DIGIT1,dp1 + dig1);
	dig0 = ClockMin - dig1*10;
	maxTransfer(MAX7221_DIGIT0,MAX7221_DP_OFF + dig0);
	DPRINT("updateDisplay:");DPRINT(dig3);DPRINT(dig2);DPRINT(":");DPRINT(dig1);DPRINTLN(dig0);
	// set the intensity of the display
	DPRINT("Intensity: ");DPRINTLN(CalcIntensity());
	maxTransfer(MAX7221_INTENSITY,CalcIntensity());
}
// ***************************************
// *** Calculate the intensity setting ***
// ***************************************
// set the intensity of the clock depending on current hour and month of year
// MaxIntensityValue is at MaxIntensityHour. This is 11:00 for Jan and 05:00 for Jul.
// MinIntensityValue is at MinIntensityHour. This is 17:00 for Jan and 23:00 for Jul.
// ClockHour is used as a ref to determine intensity:
// Intensity is switched to MaxIntensityValue when after MaxIntensityHour
// Intensity is switched to MinIntensityValue when after MinIntensityHour
byte CalcIntensity() {
	byte MaxIntensityHour;
	byte MinIntensityHour;
	byte RelIntensityHour;
	if (CurrentMonth==0) {
		// use average values if no data from time server
		MaxIntensityHour = 9;
		MinIntensityHour = 21;
	} else {
		MaxIntensityHour = 5 + abs(CurrentMonth-7);
		MinIntensityHour = 23 - abs(CurrentMonth-7);
	}
	if ( (ClockHour>=MaxIntensityHour) && (ClockHour<=MinIntensityHour) ) {
		return MaxIntensityValue;
	} else {
		return MinIntensityValue;
	}
}
