/* 
Project: wall clock using internet sync
Ron Jut, Feb 2021, v4

This sketch reads the time from a website server (not an NCP server) and shows it on a 4 digit clock display.
The reason to use a local website server it that it already corrects for day time saving.

Principle used:
- At startup it connects to local WiFi and fetches the time from website server and writes to clock.
- Using the website time, it updates the clock for the next min at hh:mm+1:00
- Consequent min updates are done using internal clock at 60 sec.
- Every hour (60 min from sync with website) the clock is synchronized with the website server again
- Time fetch is done around hh:mm:30 to avoid irregular min updates due to timing variations.
- If no valid dat received from website then clock continues running on internal clock
  Another attempt to read time data is done an hour later.
- If no local WiFi connection anymore then clock continues running on internal clock
  Another attempt to connect to local WiFi is done every minute.

Arduino controller used is NodeMCU v1 i.e. ESP-12 rev E, board library 2.6.1.
An MAX7221 chip is used to drive the clocks 4 7 segment displays.
The NodeMCU communicates with the MAX7221 using SPI and SS (Slave Select signal)
	NodeMCU	->	MAX7221
	D5|CLK		CLK|pin13
	D7|MOSI 	DIN|pin1
	D8|SS 		LOAD/CS|pin12.

Version history:
	v4	2021-02-23 : Intensity of clock is switch during day/night mode.
					 Added buttons to manually increment hour and minute (if no time from server at startup)
					 Added button to restart the ESP8266 (hardwired connected to RST pin)
	v3	2021-02-20 : Clock keeps running (only on internal clock) if WiFi lost or no time date received.
  					 ESP internal timer seems accurate to within few seconds per day!
	v2	2021-02-12 : Added code for connecting to local WiFi and fetch time from server.
	v1	2021-02-06 : Proto version. Comms NodeMCU -> MAX7221 working. Only 1 7 segment display used.
*/
// include libraries
#include "ESP8266WiFi.h"
#include "spi.h"

// debugging macros
#define DEBUG				// Remove comment for DEBUG mode!
#ifdef DEBUG
	#define DPRINT(...)		Serial.print(__VA_ARGS__)     
	#define DPRINTLN(...)	Serial.println(__VA_ARGS__)   
#else
	#define DPRINT(...)		//now defines a blank line i.e. no print output
	#define DPRINTLN(...)	//now defines a blank line i.e. no print output
#endif
